import { PaginationData } from "@/types/pagination";
import { Button, Flex, Text, useColorModeValue } from "@chakra-ui/react";
import { useSearchParams, useRouter } from "next/navigation";
import { FaAngleDoubleLeft, FaAngleDoubleRight } from "react-icons/fa";
import { FaAngleLeft, FaAngleRight } from "react-icons/fa6";

interface PaginationProps {
  tableInstance: PaginationData;
}

export default function Pagination({ tableInstance }: PaginationProps) {
  const router = useRouter();
  const searchParams = useSearchParams();

  const textColor = useColorModeValue("secondaryGray.900", "white");
  const pageCount: number = tableInstance?.last_page || 1;

  const handlePagination = (page: number) => {
    const currentParams = new URLSearchParams(searchParams.toString());
    currentParams.set("page", page.toString());
    const newUrl = `?${currentParams.toString()}`;
    router.push(newUrl);
  };

  // Generates the page numbers to display
  const generatePageNumbers = () => {
    const pages: (number | string)[] = [];
    const currentPage = tableInstance?.current_page;

    // Always show the first page
    pages.push(1);

    // Ellipsis after first page if currentPage >= 4
    if (currentPage >= 4) {
      pages.push("...");
    }

    // Show pages around the current page
    for (
      let i = Math.max(2, currentPage - 2);
      i <= Math.min(pageCount - 1, currentPage + 2);
      i++
    ) {
      pages.push(i);
    }

    // Ellipsis before last page if the difference is greater than 3
    if (currentPage <= pageCount - 4) {
      pages.push("...");
    }

    // Always show the last page if there's more than 1 page
    if (pageCount > 1) {
      pages.push(pageCount);
    }

    return pages;
  };

  const pageNumbers = generatePageNumbers();

  return (
    <Flex justify="center" align="center" mt="20px" gap={2}>
      <Button
        onClick={() => handlePagination(1)}
        isDisabled={tableInstance?.current_page === 1}
        size="sm"
      >
        <FaAngleDoubleLeft />
      </Button>
      {/* Previous Button */}
      <Button
        onClick={() => handlePagination(tableInstance?.current_page - 1)}
        isDisabled={tableInstance?.current_page === 1}
        size="sm"
      >
        <FaAngleLeft />
      </Button>

      {/* Page Numbers */}
      {pageNumbers.map((page, index) =>
        typeof page === "number" ? (
          <Button
            key={index}
            onClick={() => handlePagination(page)}
            isActive={tableInstance?.current_page === page}
            colorScheme={
              tableInstance?.current_page === page ? "brand" : "gray"
            }
            size="sm"
          >
            {page}
          </Button>
        ) : (
          <Text key={index} color={textColor} fontSize="14px" mx={1}>
            {page}
          </Text>
        )
      )}

      {/* Next Button */}
      <Button
        onClick={() => handlePagination(tableInstance?.current_page + 1)}
        isDisabled={tableInstance?.current_page === pageCount}
        size="sm"
      >
        <FaAngleRight />
      </Button>
      <Button
        onClick={() => handlePagination(pageCount)}
        isDisabled={tableInstance?.current_page === pageCount}
        size="sm"
      >
        <FaAngleDoubleRight />
      </Button>
    </Flex>
  );
}
